package cn.skcks.docking.wx.orm.generator;

import cn.hutool.core.date.DatePattern;
import com.baomidou.mybatisplus.generator.engine.VelocityTemplateEngine;
import lombok.RequiredArgsConstructor;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.springframework.boot.SpringApplication;
import org.springframework.boot.autoconfigure.jdbc.DataSourceProperties;
import org.springframework.context.ConfigurableApplicationContext;
import org.springframework.context.SmartLifecycle;
import org.springframework.context.annotation.Profile;
import org.springframework.stereotype.Service;

import com.baomidou.mybatisplus.generator.FastAutoGenerator;

import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.nio.file.Path;
import java.nio.file.Paths;

@Slf4j
@Profile("generator")
@Service
@RequiredArgsConstructor
public class OrmGenerator implements SmartLifecycle {
    private final DataSourceProperties dataSourceProperties;
    private final ConfigurableApplicationContext context;
    private boolean isRunning;

    private final static String sourceDir = Paths.get(System.getProperty("user.dir")) + "/platform-orm/src/main/java";

    @Override
    public void start() {
        isRunning = true;

        generate();
    }

    @Override
    public void stop() {
        isRunning = false;
    }

    @Override
    public boolean isRunning() {
        return isRunning;
    }

    @SneakyThrows
    private static String getGitUserName(){
        BufferedReader bufferedReader = new BufferedReader(
                new InputStreamReader(
                        Runtime.getRuntime().exec("git config user.name")
                                .getInputStream()));
        return bufferedReader.readLine();
    }

    private void generate(){
        log.info("[+] 开始从数据库生成实体类");
        FastAutoGenerator.create(dataSourceProperties.getUrl(), dataSourceProperties.getUsername(), dataSourceProperties.getPassword())
                .globalConfig(builder -> {
                    builder.author(getGitUserName())
                            .disableOpenDir()
                            .enableSpringdoc()
                            .outputDir(sourceDir)
                            .commentDate(DatePattern.NORM_DATE_PATTERN);
                })
                .packageConfig(builder -> {
                    builder.parent("cn.skcks.docking.wx.orm")
                            .entity("entity")
                            .mapper("mapper")
                            .service("service")
                            .serviceImpl("service.impl")
                            .xml("mapper.xml");
                }).strategyConfig(builder -> builder
                        .enableSkipView()
                        .addInclude("^mp_.*")
                        .entityBuilder()
                        //开启文件覆盖
                        // .enableFileOverride()
                        //开启链式模型
                        .enableChainModel()
                        //开启Lombok
                        .enableLombok()
                        .enableTableFieldAnnotation()//开启生成实体时生成字段注解
                        //开启 ActiveRecord 模型(封装了实体一些简单持久化操作,必须有对应的BaseMapper实现)
                        .enableActiveRecord()
                        .controllerBuilder().disable()
                        .serviceBuilder().disable()
                )
                .templateEngine(new VelocityTemplateEngine())
                .execute();
        System.exit(SpringApplication.exit(context, () -> 0));
    }
}
