package cn.skcks.docking.wx.services.mp.controller;

import cn.skcks.docking.wx.annotation.web.methods.GetJson;
import cn.skcks.docking.wx.common.json.JsonResponse;
import cn.skcks.docking.wx.common.page.PageWrapper;
import cn.skcks.docking.wx.orm.entity.MpApp;
import cn.skcks.docking.wx.services.mp.convertor.MpAppConvertor;
import cn.skcks.docking.wx.services.mp.dto.app.MpAppDto;
import cn.skcks.docking.wx.services.mp.service.app.MpAppService;
import cn.skcks.docking.wx.services.mp.vo.verify.SignatureVo;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.Parameters;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpServletResponse;
import lombok.RequiredArgsConstructor;
import org.springdoc.core.annotations.ParameterObject;
import org.springframework.web.bind.annotation.*;

import java.util.List;

@Tag(name = "微信公众号")
@RequestMapping("/mp-app")
@RestController
@RequiredArgsConstructor
public class MpAppController {
    private final MpAppService mpAppService;


    @Operation(summary = "公众号列表")
    @GetJson("/list")
    public JsonResponse<List<MpAppDto>> getMpAppList() {
        List<MpApp> mpAppList = mpAppService.getMpAppList();
        return JsonResponse.success(MpAppConvertor.INSTANCE.toMpAppDto(mpAppList));
    }

    @Operation(summary = "公众号分页列表")
    @GetJson("/page")
    @Parameters({
            @Parameter(name = "pageNo", description = "页数", example = "1"),
            @Parameter(name = "pageSize", description = "条数", example = "10")
    })
    public JsonResponse<PageWrapper<MpAppDto>> getMpAppListPage(@RequestParam Integer pageNo, @RequestParam Integer pageSize) {
        return JsonResponse.success(MpAppConvertor.INSTANCE.toMpAppDto(mpAppService.getMpAppPageList(pageNo, pageSize)));
    }

    @RequestMapping("/{appId}")
    public void verify(HttpServletRequest request, HttpServletResponse response,
                           @PathVariable String appId, @ParameterObject SignatureVo signatureVo) throws Exception {
        mpAppService.service(request, response, appId, signatureVo);
    }
}
