package cn.skcks.docking.wx.services.mp.service.message.handler;

import cn.skcks.docking.wx.services.mp.vo.verify.SignatureVo;
import jakarta.annotation.Nullable;
import jakarta.servlet.http.HttpServletRequest;
import lombok.SneakyThrows;
import me.chanjar.weixin.mp.api.WxMpService;
import me.chanjar.weixin.mp.bean.message.WxMpXmlMessage;
import org.springframework.beans.BeansException;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.context.ApplicationContext;
import org.springframework.context.ApplicationContextAware;
import org.springframework.stereotype.Service;

import java.util.Collection;
import java.util.Comparator;
import java.util.LinkedList;

@Service
public class MessageHandlerService implements InitializingBean, ApplicationContextAware {
    private ApplicationContext applicationContext;
    private final Collection<MessageHandler> messageHandlers = new LinkedList<>();

    @Override
    public void setApplicationContext(ApplicationContext applicationContext) throws BeansException {
        this.applicationContext = applicationContext;
    }

    @Override
    public void afterPropertiesSet() throws Exception {
        applicationContext.getBeansOfType(MessageHandler.class)
                .values()
                .stream()
                .sorted(Comparator.comparingInt(MessageHandler::order))
                .forEach(messageHandlers::add);
    }

    @Nullable
    @SneakyThrows
    public WxMpXmlMessage handleMessage(HttpServletRequest request, WxMpService wxMpService, SignatureVo signatureVo) {
        for (MessageHandler messageHandler : messageHandlers) {
            if(messageHandler.canHandle(request)){
                return messageHandler.receive(request, wxMpService, signatureVo);
            }
        }

        return null;
    }
}
